
SET SQL DIALECT 3; 

/* CREATE DATABASE 'Ancestromania-data/var/lib/firebird/3.0/data/Parances.fdb' PAGE_SIZE 4096 DEFAULT CHARACTER SET UTF8; */


/*  Generators or sequences */
CREATE GENERATOR GEN_REF_DEPARTEMENTS;
CREATE GENERATOR GEN_REF_HISTOIRE;
CREATE GENERATOR GEN_REF_PAYS;
CREATE GENERATOR GEN_REF_REGIONS;

COMMIT WORK;

/* Table: PARAMETRES, Owner: SYSDBA */
CREATE TABLE PARAMETRES (GROUPE VARCHAR(25) CHARACTER SET ASCII NOT NULL,
        PARAMETRE VARCHAR(25) CHARACTER SET ASCII NOT NULL,
        VALEUR VARCHAR(255),
CONSTRAINT PK_PARAMETRES PRIMARY KEY (GROUPE, PARAMETRE));

/* Table: REF_CP_VILLES, Owner: SYSDBA */
CREATE TABLE REF_CP_VILLES (RCV_CODE INTEGER NOT NULL,
        RCV_POSTE VARCHAR(8),
        RCV_VILLE VARCHAR(50),
        RCV_DEPT INTEGER,
        RCV_REGION INTEGER,
        RCV_PAYS INTEGER,
        RCV_INSEE VARCHAR(6),
        RCV_LATITUDE DECIMAL(11, 8),
        RCV_LONGITUDE DECIMAL(11, 8),
        RCV_DIVERS VARCHAR(90),
        RCV_VILLE_MAJ VARCHAR(50),
CONSTRAINT PK_REF_CP_VILLES PRIMARY KEY (RCV_CODE));

/* Table: REF_DEPARTEMENTS, Owner: SYSDBA */
CREATE TABLE REF_DEPARTEMENTS (RDP_CODE INTEGER NOT NULL,
        RDP_LIBELLE VARCHAR(30),
        RDP_REGION INTEGER,
        RDP_PAYS INTEGER,
CONSTRAINT PK_REF_DEPARTEMENTS PRIMARY KEY (RDP_CODE));

/* Table: REF_HISTOIRE, Owner: SYSDBA */
CREATE TABLE REF_HISTOIRE (HI_ID INTEGER NOT NULL,
        HI_DOSSIER INTEGER DEFAULT 0,
        HI_DICORIGINE VARCHAR(8),
        HI_DATE_TEXTE VARCHAR(50) NOT NULL,
        HI_CAT INTEGER,
        HI_TITRE VARCHAR(50),
        HI_TEXTE BLOB SUB_TYPE TEXT SEGMENT SIZE 80,
        HI_IMAGE BLOB SUB_TYPE 0 SEGMENT SIZE 80,
        HI_DATE_CODE_DEBUT INTEGER,
        HI_DATE_CODE_FIN INTEGER,
CONSTRAINT PK_REF_HISTOIRE PRIMARY KEY (HI_ID));

/* Table: REF_PAYS, Owner: SYSDBA */
CREATE TABLE REF_PAYS (RPA_CODE INTEGER NOT NULL,
        RPA_LIBELLE VARCHAR(30) NOT NULL,
        RPA_ABREVIATION VARCHAR(3),
CONSTRAINT PK_REF_PAYS PRIMARY KEY (RPA_CODE));

/* Table: REF_REGIONS, Owner: SYSDBA */
CREATE TABLE REF_REGIONS (RRG_CODE INTEGER NOT NULL,
        RRG_LIBELLE VARCHAR(30),
        RRG_PAYS INTEGER,
CONSTRAINT PK_REF_REGIONS PRIMARY KEY (RRG_CODE));

COMMIT WORK;
SET AUTODDL OFF;
SET TERM ^ ;

/* Stored procedures headers */
CREATE OR ALTER PROCEDURE F_MAJ_SANS_ACCENT (S_IN VARCHAR(255) CHARACTER SET UTF8)
RETURNS (S_OUT VARCHAR(255) CHARACTER SET UTF8)
AS 
BEGIN SUSPEND; END ^
CREATE OR ALTER PROCEDURE PROC_PREP_VILLES_RAYON (LIMITE DOUBLE PRECISION,
LATITUDEA DOUBLE PRECISION,
LONGITUDEA DOUBLE PRECISION)
RETURNS (CP_CODE INTEGER,
CP_CP VARCHAR(8) CHARACTER SET UTF8,
CP_VILLE VARCHAR(103) CHARACTER SET UTF8,
CP_DEPT INTEGER,
CP_REGION INTEGER,
CP_PAYS INTEGER,
CP_INSEE VARCHAR(6) CHARACTER SET UTF8,
CP_DIVERS VARCHAR(90) CHARACTER SET UTF8,
CP_LATITUDE DOUBLE PRECISION,
CP_LONGITUDE DOUBLE PRECISION,
DISTANCE DOUBLE PRECISION,
DEPARTEMENT VARCHAR(30) CHARACTER SET UTF8,
REGION VARCHAR(50) CHARACTER SET UTF8,
PAYS VARCHAR(30) CHARACTER SET UTF8)
AS 
BEGIN SUSPEND; END ^

SET TERM ; ^
COMMIT WORK;
SET AUTODDL ON;

/*  Index definitions for all user tables */
CREATE INDEX IDX_REF_CP_VILLES_INSEE ON REF_CP_VILLES (RCV_INSEE);
CREATE INDEX IDX_REF_CP_VILLES_LATITUDE ON REF_CP_VILLES (RCV_LATITUDE);
CREATE INDEX IDX_REF_CP_VILLES_LONGITUDE ON REF_CP_VILLES (RCV_LONGITUDE);
CREATE INDEX IDX_REF_CP_VILLES_POSTE ON REF_CP_VILLES (RCV_POSTE);
CREATE INDEX IDX_REF_CP_VILLES_VILLE ON REF_CP_VILLES (RCV_VILLE);
CREATE INDEX IDX_REF_CP_VILLES_VILLE_MAJ ON REF_CP_VILLES (RCV_VILLE_MAJ);
CREATE INDEX IDX_REF_DEPARTEMENTS_LIBELLE ON REF_DEPARTEMENTS (RDP_LIBELLE);
CREATE INDEX IDX_REF_HISTOIRE_DEBUT ON REF_HISTOIRE (HI_DATE_CODE_DEBUT);
CREATE DESCENDING INDEX IDX_REF_HISTOIRE_DEBUT_DESC ON REF_HISTOIRE (HI_DATE_CODE_DEBUT);
CREATE INDEX IDX_REF_HISTOIRE_DICORIGINE ON REF_HISTOIRE (HI_DICORIGINE);
CREATE INDEX IDX_REF_HISTOIRE_DOSSIER ON REF_HISTOIRE (HI_DOSSIER);
CREATE INDEX IDX_REF_HISTOIRE_FIN ON REF_HISTOIRE (HI_DATE_CODE_FIN);
CREATE INDEX IDX_REF_PAYS_LIBELLE ON REF_PAYS (RPA_LIBELLE);
CREATE INDEX IDX_REF_REGIONS_LIBELLE ON REF_REGIONS (RRG_LIBELLE);

ALTER TABLE REF_CP_VILLES ADD CONSTRAINT FK_REF_CP_VILLES_DEPT FOREIGN KEY (RCV_DEPT) REFERENCES REF_DEPARTEMENTS (RDP_CODE) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE REF_CP_VILLES ADD CONSTRAINT FK_REF_CP_VILLES_PAYS FOREIGN KEY (RCV_PAYS) REFERENCES REF_PAYS (RPA_CODE) ON UPDATE CASCADE;

ALTER TABLE REF_CP_VILLES ADD CONSTRAINT FK_REF_CP_VILLES_REGION FOREIGN KEY (RCV_REGION) REFERENCES REF_REGIONS (RRG_CODE) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE REF_DEPARTEMENTS ADD CONSTRAINT FK_REF_DEPARTEMENTS_PAYS FOREIGN KEY (RDP_PAYS) REFERENCES REF_PAYS (RPA_CODE) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE REF_DEPARTEMENTS ADD CONSTRAINT FK_REF_DEPARTEMENTS_REGION FOREIGN KEY (RDP_REGION) REFERENCES REF_REGIONS (RRG_CODE) ON UPDATE CASCADE ON DELETE CASCADE;

ALTER TABLE REF_REGIONS ADD CONSTRAINT FK_REF_REGIONS FOREIGN KEY (RRG_PAYS) REFERENCES REF_PAYS (RPA_CODE) ON UPDATE CASCADE ON DELETE CASCADE;

COMMIT WORK;
SET AUTODDL OFF;
SET TERM ^ ;

/* Stored procedures bodies */

ALTER PROCEDURE F_MAJ_SANS_ACCENT (S_IN VARCHAR(255) CHARACTER SET UTF8)
RETURNS (S_OUT VARCHAR(255) CHARACTER SET UTF8)
AS 
declare variable i integer;
declare variable l integer;
declare variable car char(1);
declare variable s_temp varchar(255);
begin
  s_temp=upper(lower(S_IN));
  l=char_length(s_temp);
  s_out='';
  i=1;
  while (i<=l) do
  begin
    car=substring(s_temp from i for 1);
    s_out=case car
      when 'Å“' then s_out||'OE'
      when 'Å’' then s_out||'OE'
      when 'Ã†' then s_out||'AE'
      when 'Ã„' then s_out||'A'
      when 'Ãƒ' then s_out||'A'
      when 'Ã–' then s_out||'O'
      when 'Ã•' then s_out||'O'
      when 'Ã‘' then s_out||'N'
      when 'Ã' then s_out||'Y'
      when 'Ã' then s_out||'D'
      when 'Ã˜' then s_out||'O'
      when 'Ãœ' then s_out||'U'
      when 'Ã‡' then s_out||'C'
    else s_out||car
    end;
    i=i+1;
  end
  suspend;
end ^

ALTER PROCEDURE PROC_PREP_VILLES_RAYON (LIMITE DOUBLE PRECISION,
LATITUDEA DOUBLE PRECISION,
LONGITUDEA DOUBLE PRECISION)
RETURNS (CP_CODE INTEGER,
CP_CP VARCHAR(8) CHARACTER SET UTF8,
CP_VILLE VARCHAR(103) CHARACTER SET UTF8,
CP_DEPT INTEGER,
CP_REGION INTEGER,
CP_PAYS INTEGER,
CP_INSEE VARCHAR(6) CHARACTER SET UTF8,
CP_DIVERS VARCHAR(90) CHARACTER SET UTF8,
CP_LATITUDE DOUBLE PRECISION,
CP_LONGITUDE DOUBLE PRECISION,
DISTANCE DOUBLE PRECISION,
DEPARTEMENT VARCHAR(30) CHARACTER SET UTF8,
REGION VARCHAR(50) CHARACTER SET UTF8,
PAYS VARCHAR(30) CHARACTER SET UTF8)
AS 
declare variable degrad double precision;
declare variable sinlat double precision;
declare variable coslat double precision;
declare variable deltalat double precision;
declare variable deltalong double precision;
declare variable latmax double precision;
declare variable latmin double precision;
declare variable longmax double precision;
declare variable longmin double precision;
declare variable ok integer;
begin
degrad=pi()/180;
sinlat=sin(latitudea*degrad);
coslat=cos(latitudea*degrad);
deltalat=limite/6367;
latmax=latitudea+deltalat/degrad;
latmin=latitudea-deltalat/degrad;
if (latmax>90 or latmin<-90) then
begin
   longmax=180;
   longmin=-180;
end
else
begin
   deltalong=asin(deltalat/coslat)/degrad;
   longmax=longitudea+deltalong;
   longmin=longitudea-deltalong;
end
for
select rcv_code,
  rcv_poste,
  rcv_ville,
  rcv_dept,
  rcv_region,
  rcv_pays,
  rcv_insee,
  rcv_divers,
  rcv_latitude,
  rcv_longitude
from ref_cp_villes
where rcv_latitude between cast(:latmin as decimal(11,8)) and cast(:latmax as decimal(11,8))
  and (rcv_longitude between cast(:longmin as decimal(11,8)) and cast(:longmax as decimal(11,8))
  or rcv_longitude between cast(:longmin+360 as decimal(11,8)) and cast(180 as decimal(11,8))
  or rcv_longitude between cast(-180 as decimal(11,8)) and cast(:longmax-360 as decimal(11,8)))
into :cp_code,
  :cp_cp,
  :cp_ville,
  :cp_dept,
  :cp_region,
  :cp_pays,
  :cp_insee,
  :cp_divers,
  :cp_latitude,
  :cp_longitude
  do
  begin
    ok=0;
    if (cp_latitude=latitudea and cp_longitude=longitudea) then
    begin
      distance=0;
      ok=1;
    end
    else
    begin
      distance=6367*acos(sinlat*sin(cp_latitude*degrad)+coslat*cos(cp_latitude*degrad)*cos((longitudea-cp_longitude)*degrad));
      if (distance<=limite) then
        ok=1;
    end
    if (ok=1) then
    begin
      select first(1) rdp_libelle from ref_departements where rdp_code=:cp_dept
        into :departement;
      select first(1) rrg_libelle from ref_regions where rrg_code=:cp_region
        into :region;
      select first(1) rpa_libelle from ref_pays where rpa_code=:cp_pays
        into :pays;
      suspend;
    end
  end
end ^

SET TERM ; ^
COMMIT WORK;
SET AUTODDL ON;
SET TERM ^ ;

/* Triggers only will work for SQL triggers */
CREATE TRIGGER PARAMETRES_BIU FOR PARAMETRES 
ACTIVE BEFORE INSERT OR UPDATE POSITION 0 
as
begin
  new.parametre=upper(new.parametre);
  new.groupe=upper(new.groupe);
  new.valeur=trim(new.valeur);
end ^

CREATE TRIGGER REF_DEPARTEMENTS_BIU FOR REF_DEPARTEMENTS 
ACTIVE BEFORE INSERT OR UPDATE POSITION 0 
as
begin
  if (new.rdp_code is null) then
    new.rdp_code=gen_id(gen_ref_departements,1);
end ^

CREATE TRIGGER REF_HISTOIRE_BI FOR REF_HISTOIRE 
ACTIVE BEFORE INSERT POSITION 0 
AS
BEGIN
  IF (NEW.HI_ID IS NULL) THEN 
    NEW.HI_ID = GEN_ID(GEN_REF_HISTOIRE, 1);
END ^

CREATE TRIGGER REF_PAYS_BIU FOR REF_PAYS 
ACTIVE BEFORE INSERT OR UPDATE POSITION 0 
as
begin
  if (new.rpa_code is null) then
    new.rpa_code=gen_id(gen_ref_pays,1);
end ^

CREATE TRIGGER REF_REGIONS_BIU FOR REF_REGIONS 
ACTIVE BEFORE INSERT OR UPDATE POSITION 0 
as
begin
  if (new.rrg_code is null) then
    new.rrg_code=gen_id(gen_ref_regions,1);
end ^


SET TERM ; ^
COMMIT WORK;

/* Grant permissions for this database */
GRANT USAGE ON SEQUENCE GEN_REF_DEPARTEMENTS TO PUBLIC;
GRANT USAGE ON SEQUENCE GEN_REF_HISTOIRE TO PUBLIC;
GRANT USAGE ON SEQUENCE GEN_REF_PAYS TO PUBLIC;
GRANT USAGE ON SEQUENCE GEN_REF_REGIONS TO PUBLIC;

/* Comments for database objects. */
COMMENT ON PROCEDURE    PROC_PREP_VILLES_RAYON IS 'crÃ©ation par AndrÃ©
Utilisations: Villes voisines Ancestromania et Cassinivision';
